/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * colors.cc: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "colors.h"
/*-----------------------------------------------------------------*/
#define Uses_TColorGroup
#define Uses_TColorItem
#define Uses_TColorDialog
#define Uses_TPalette
#define Uses_TDeskTop
#define Uses_TApplication
#define Uses_TChDirDialog
#define Uses_TScreen

#include <tv.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "main.h"
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void Colors()
{
 TColorsObj *CO;

 CO = new TColorsObj();
 CO->Run();
 DCApp->destroy(CO);
}
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
//                 TColorsObj : public TObject
/*-----------------------------------------------------------------*/
TColorsObj::TColorsObj():TObject()
{
}
/*-----------------------------------------------------------------*/
void TColorsObj::shutDown()
{
 TObject::shutDown();
}
/*-----------------------------------------------------------------*/
void TColorsObj::Run()
{
    TColorGroup &group1 =
        *new TColorGroup("Desktop") +
            *new TColorItem("Color",             1)+

        *new TColorGroup("Menus") +
            *new TColorItem("Normal",            2)+
            *new TColorItem("Disabled",          3)+
            *new TColorItem("Shortcut",          4)+
            *new TColorItem("Selected",          5)+
            *new TColorItem("Selected disabled", 6)+
            *new TColorItem("Shortcut selected", 7);
            
    TColorGroup &group2 =
        *new TColorGroup("Dialogs") +
            *new TColorItem("Frame/background",  33)+
            *new TColorItem("Frame icons",       34)+
            *new TColorItem("Scroll bar page",   35)+
            *new TColorItem("Scroll bar icons",  36)+
            *new TColorItem("Static text",       37)+

            *new TColorItem("Label normal",      38)+
            *new TColorItem("Label selected",    39)+
            *new TColorItem("Label shortcut",    40);

    TColorItem &item_coll1 =
        *new TColorItem("Button normal",     41)+
        *new TColorItem("Button default",    42)+
        *new TColorItem("Button selected",   43)+
        *new TColorItem("Button disabled",   44)+
        *new TColorItem("Button shortcut",   45)+
        *new TColorItem("Button shadow",     46)+
        *new TColorItem("Cluster normal",    47)+
        *new TColorItem("Cluster selected",  48)+
        *new TColorItem("Cluster shortcut",  49);

    TColorItem &item_coll2 =
        *new TColorItem("Input normal",      50)+
        *new TColorItem("Input selected",    51)+
        *new TColorItem("Input arrow",       52)+

        *new TColorItem("History button",    53)+
        *new TColorItem("History sides",     54)+
        *new TColorItem("History bar page",  55)+
        *new TColorItem("History bar icons", 56)+

        *new TColorItem("List normal",       57)+
        *new TColorItem("List focused",      58)+
        *new TColorItem("List selected",     59)+
        *new TColorItem("List divider",      60);

     group2 = group2 + item_coll1 + item_coll2;

     TColorGroup &group5 = group1 + group2;

        TPalette* temp_pal = new TPalette(DCApp->getPalette());
    TColorDialog* c        = new TColorDialog(temp_pal, &group5 );

    if ( DCApp->validView( c ) != 0 ) {
      c->helpCtx = 100;
      c->setData(&DCApp->getPalette());
      if( DCApp->deskTop->execView( c ) != cmCancel ) {
         DCApp->getPalette() = *(c->pal);
         DCApp->setScreenMode(TScreen::screenMode);

         unlink(colCfgFile);
         int x = creat(colCfgFile, S_IREAD|S_IWRITE);
         close(x);
         x = open(colCfgFile, O_WRONLY);
         if (x!=-1) {
            uchar* p1 = DCApp->getPalette().data;
           ushort  len = p1[0];
           write(x, p1, len+1);
           close(x);
         }
      }
      destroy( c );
    }
    delete temp_pal;
}
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
void SetColors()
{
  int x;
  
  x = open(colCfgFile, O_RDONLY);
  if (x==-1) return;

  uchar ps;
  char  p[256];
  read(x, &ps, 1);
  read(x, p, ps-1);
  close(x);

  TPalette *pal = new TPalette(p, ps);
  DCApp->getPalette() = *(pal);
  delete pal;
}
/*-----------------------------------------------------------------*/

