/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * inifile.cc: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "inifile.h"

/*---------------------------------------------------------------*/
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>

#include "tools.h"
/*---------------------------------------------------------------*/


/*---------------------------------------------------------------*/
TINIfile::TINIfile():TObject()
{
 mode       = 0;
 fd         = NULL;
 inisection = NULL;
 iniitem    = NULL;
 fullname   = false;
}
/*---------------------------------------------------------------*/
TINIfile::TINIfile(bool Afullname):TObject()
{
 mode       = 0;
 fd         = NULL;
 inisection = NULL;
 iniitem    = NULL;
 fullname   = Afullname;
}
/*---------------------------------------------------------------*/
void TINIfile::shutDown()
{
 DelStr(inisection);
 DelStr(iniitem);
 TObject::shutDown();
}
/*---------------------------------------------------------------*/
bool TINIfile::Read(char* file, char* section, char* item, char* value, char* defaultValue)
{
 char    buf[256];
 char    line[256];
 int     ok;
 
 value[0]=0;

 DelStr(inisection); inisection = NewStr(section);
 DelStr(iniitem);    iniitem    = NewStr(item);
 
 Upper(section, inisection);
 Lower(item, iniitem);
 
 strcpy(buf, file);
 if (!fullname) strcat(buf,".ini");
 fd = fopen(buf, "r");
 if ( fd == NULL ) {
   strcpy(value, defaultValue);
   return false;
 }
 sprintf(buf, "[%s]", inisection); ok = 0;
 while (fgets(line, 255, fd)!=NULL) {
   Upper2(line);
   if (strstr(line, buf)!=0) {
     ok=1;
     break;
   }
 }
 if (ok==0) {
   fclose(fd);
   strcpy(value, defaultValue);
   return false;
 }
 
 sprintf(buf, "%s=", iniitem); ok = 0;
 while (fgets(line, 255, fd)!=NULL) {
   if (line[0]=='[') break;
   if (strstr(line, buf)!=0) {
     ok=1;
     break;
   }
 }
 fclose(fd);
 if (ok==0) {
   strcpy(value, defaultValue);
   return false;
 }
 
 int len;
 len = strlen(buf);
 strcpy(value, (char*)(line+len));
 value[strlen(value)-1]=0;
 return true;
}
/*---------------------------------------------------------------*/
bool TINIfile::Read(char* file, char* section, long item, char* value, char *defaultValue)
{
 char it[25];
 sprintf(it,"%ld", item);
 return Read(file, section, it, value, defaultValue);
}
/*---------------------------------------------------------------*/
bool TINIfile::Write(char* file, char* section, char* item, char* value)
{
 char    buf[256];
 char    line[256];
 FILE*   fd1;
 int     ret;
 int     ok;
 
 DelStr(inisection); inisection = NewStr(section);
 DelStr(iniitem);    iniitem    = NewStr(item);

 Upper(section, inisection);
 Lower(item, iniitem);

 strcpy(buf,file);
 if (!fullname) strcat(buf,".ini");
 fd=fopen(buf, "r");
 if ( fd == NULL ) {
   fd = fopen(buf, "a");
   fclose(fd);
   fd = fopen(buf, "r");
 }
 if ( fd == NULL ) return false;
 
 sprintf(buf, "%s.ini.tmp", file);  
 unlink(buf);
 fd1 = fopen(buf, "a");
 if ( fd1 == NULL ) return false;
 
 sprintf(buf, "[%s]\n", inisection);
 ok = 0;
 while(fgets(line, 255, fd)!=NULL) {
   fprintf(fd1, "%s", line);
   if (strstr(line, buf)!=0) {
     ok=1;
     break;
   }
 }
 
 if (ok==0) {
   if (mode==0) {
     fprintf(fd1, "[%s]\n", inisection);
     fprintf(fd1, "%s=%s\n", iniitem, value);
   }
 
   } else {
   sprintf(buf, "%s=", iniitem);
   ok = 0;
   while (fgets(line, 255, fd)!=NULL) {
     if (line[0]=='[') { 
       if (mode==0)
          fprintf(fd1, "%s=%s\n", iniitem,value);
       fprintf(fd1, "%s", line); ok=1;
       break; 
     }
     if (strstr(line, buf)!=0) { 
       if (mode==0)
         fprintf(fd1, "%s=%s\n", iniitem,value);
       ok=1;
       break;
     }
     fprintf(fd1, "%s", line);
   }
   if ((mode==0) && (ok==0))
      fprintf(fd1, "%s=%s\n", iniitem,value);
   while (fgets(line, 255, fd)!=NULL) {
     fprintf(fd1, "%s", line);
   }
 }

 fclose(fd);
 fclose(fd1);
 
 strcpy(buf,file);
 if (!fullname) strcat(buf,".ini");
 sprintf(line, "%s.ini.tmp", file);
 rename(line,buf);
 chmod(buf, 33200);
 
 return true;
}
/*---------------------------------------------------------------*/
bool TINIfile::Write(char* file, char* section, long item, char* value)
{
 char b[25];
 sprintf(b,"%ld", item);
 return Write(file, section, b, value);
}
/*---------------------------------------------------------------*/
bool TINIfile::Delete(char* file, char* section, long item)
{
 char b[25];
 sprintf(b,"%ld", item);
 return Delete(file, section, b);
}
/*---------------------------------------------------------------*/
bool TINIfile::Delete(char* file, char* section, char* item)
{
 bool ret;

 mode = 1;
 ret=Write(file,section,item, " ");
 mode = 0;
 return ret;
}
/*---------------------------------------------------------------*/
void TINIfile::Upper(char* strin, char* strout)
{
 int n, n1;
 
 n1 = strlen(strin);
 for(n=0; n<n1; n++)
    strout[n]=toupper(strin[n]);
}
/*---------------------------------------------------------------*/
void TINIfile::Upper2(char* strin)
{
 char* p1 = strin;
 
 while (*p1!='\0') {
   *p1 = toupper( *p1 );
   p1++;
 }
}
/*---------------------------------------------------------------*/
void TINIfile::Lower(char* strin, char* strout)
{
 int n, n1;
 n1 = strlen(strin);
 for (n=0; n<n1; n++)
   strout[n]=tolower(strin[n]);
}
/*---------------------------------------------------------------*/

