/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * main.cc: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

//
// LDCC2 - linux Direct Connect Client
//         with TurboVision interface
//
// (C) 2002-2003 Marcin Slonicki
// http://www.softservice.com.pl/ldcc
// marcins@softservice.com.pl
//

#define Uses_TApplication
#define Uses_TMenuBar
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TStatusLine
#define Uses_TStatusItem
#define Uses_TStatusDef
#define Uses_TKeys
#define Uses_TFileViewer
#define Uses_TDeskTop
#define Uses_TEvent
#define Uses_TDialog
#define Uses_TStaticText
#define Uses_TButton
#define Uses_MsgBox

#include <tv.h>
/*-----------------------------------------------------------------*/
#define VER_NO    "2.0.7"
#define VER_DATE  "Feb.26,2003"
/*-----------------------------------------------------------------*/
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <stdlib.h>

#include "main.h"
#include "const.h"
#include "hub.h"
#include "search.h"
#include "options.h"
#include "info.h"
#include "sdir.h"
#include "colors.h"
/*-----------------------------------------------------------------*/
TDCApp* DCApp;
char*   cfgFile;
char*   logFile;
char*   debFile;
char*   colCfgFile;
/*-----------------------------------------------------------------*/
//                           TDCApp
/*-----------------------------------------------------------------*/
TDCApp::TDCApp()
       :TProgInit(&TDCApp::initStatusLine,
                  &TDCApp::initMenuBar,
                  &TDCApp::initDeskTop),TApplication()
{

 char* s_tmp  = NewStr(MAXMSG);
 
 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc");
 mkdir(s_tmp, S_IRWXU);
 strcat2(s_tmp,"/GDL");
 mkdir(s_tmp, S_IRWXU);
 
 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc/ldcc.cfg");
 cfgFile = NewStr(s_tmp);
 
 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc/ldcc.log");
 logFile = NewStr(s_tmp);

 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc/ldcc_err.log");
 debFile = NewStr(s_tmp);

 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc/colors.cfg");
 colCfgFile = NewStr(s_tmp);

 DelStr(s_tmp);

 for (int i=0; i<MAX_HUB_TABLE; i++) {
   HUB_TABLE[i].No     = 0;
   HUB_TABLE[i].Dialog = NULL;
 }

 deskTop->insert(new TInfo(TRect(0,0,80,1)));

}
/*-----------------------------------------------------------------*/
TMenuBar* TDCApp::initMenuBar(TRect r)
{
 #include "menu.h"
}
/*-----------------------------------------------------------------*/
TStatusLine* TDCApp::initStatusLine(TRect r)
{
 #include "status.h"
}
/*-----------------------------------------------------------------*/
void TDCApp::handleEvent(TEvent &event)
{
 TApplication::handleEvent(event);

 if (event.what == evCommand) {
   switch (event.message.command) {
     case cmAbout:      about();        break;
//     case cmReadMe:     readMe();       break;
//     case cmCommands:   commands();     break;
//     case cmVAR:        VAR();          break;
     case cmServers:    Servers();      break;
     case cmOptions:    DCOptions();    break;
     case cmShared:     Shared();       break;
     case cmDDir:       Download();     break;
     case cmBeeper:     Beeper();       break;
     case cmCoverter:   Coverter();     break;
     case cmFavorite:   Favorite();     break;
     case cmColors:     Colors();       break;
    
     default:                           return;
   }
   clearEvent(event);
 }
}
/*-----------------------------------------------------------------*/
void TDCApp::idle()
{
 TApplication::idle();

 for (int i=0; i<MAX_HUB_TABLE; i++) {
   THUB *H = HUB_TABLE[i].Dialog;
   if (H!=NULL) H->idle();
 }
}
/*-----------------------------------------------------------------*/
void TDCApp::run()
{
 SetColors();
 
 LastServ[0]='\0';
 LastSearch[0]='\0';
 
 ReadOptions();
 if (opt.Log==1) {
   f_log = fopen(logFile, "a+");
   if (f_log==NULL) {
     messageBox("\003Can't create log file.",mfError|mfOKButton);
     opt.Log=0;
     } else {
     messageBox("\003Main chat\n\003logging enabled.", mfInformation|mfOKButton);
   }
   } else {
   f_log=NULL;
 }

 if (opt.Deb==1) {
   f_deb = fopen(debFile, "a+");
   if (f_deb==NULL) {
     messageBox("\003Can't create\n\003error/debug log file.",mfError|mfOKButton);
     opt.Deb=0;
     } else {
     messageBox("\003Errors/debug\n\003logging enabled.", mfError|mfOKButton);
   }
   } else {
   f_deb=NULL;
 }

 about();

 AutoConnect();
 
 TApplication::run();

 if (f_log!=NULL) fclose(f_log);
 if (f_deb!=NULL) fclose(f_deb);
}
/*-----------------------------------------------------------------*/
void TDCApp::AutoConnect()
{
          char* path;
           DIR* dir;
 struct dirent* obj;
           int  a;

 path = NewStr(MAXMSG);
 strcpy2(path, getenv("HOME"));
 strcat2(path, "/.dctc/running");

 dir=opendir(path);
 if(dir!=NULL) {
   while((obj=readdir(dir))!=NULL) {
     if (strncmp(obj->d_name,"dctc-",5)==0) {
       a=strlen(obj->d_name);
       if (a>9) {
         if (strcmp(obj->d_name+a-4,".udp")!=0) {
           if (strcmp(obj->d_name+a-5,".done")!=0) {
             if (strcmp(obj->d_name+a-9,".userinfo")!=0) {
               //found a socket
               char s1_tmp[81];
               strcpy2(s1_tmp, (char*)(obj->d_name+5));
               InsertHUB(s1_tmp, s1_tmp, false);
             }
           }
         }
       }
     }
   }
   } else { // if (dir!=NULL)
   messageBox("\003Unable to dctc running path", mfError|mfOKButton);
   return;
 }
 closedir(dir);
 DelStr(path);
}
/*-----------------------------------------------------------------*/
void TDCApp::about()
{
  TDialog* D;
  char*    b =  NewStr(81);

  D = new TDialog(TRect(0,0,50,15), "About");
  D->options |= ofCentered;
                                 sprintf(b, "              LDCC ver. %s", VER_NO);
  D->insert(new TStaticText(TRect(1,2,49,3),b));
  D->insert(new TStaticText(TRect(1,3,49,4),"        LINUX Direct Connect Client"));
  D->insert(new TStaticText(TRect(2,5,49,6),"         (C) 2002 Marcin Slonicki"));
  D->insert(new TStaticText(TRect(2,6,49,7),"      http://www.softservice.com.pl/ldcc"));
  D->insert(new TStaticText(TRect(2,7,49,8),"       email marcins@softservice.com.pl"));
  D->insert(new TStaticText(TRect(1,8,49,9),"------------------------------------------------"));
  D->insert(new TStaticText(TRect(2,9,49,10),"Message board:"));
  D->insert(new TStaticText(TRect(2,10,49,11),"http://www.softservice.com.pl/ldcc/board2"));
  D->insert(new TStaticText(TRect(1,11,49,12), "------------------------------------------------"));
  D->insert(new TStaticText(TRect(2,12,20,13), "FREEWARE"));
  D->insert(new TStaticText(TRect(2,13,20,14), VER_DATE));
  D->insert(new TButton(TRect(37,12,48,14),"Close",cmCancel,bfNormal));

  DelStr(b);
  
  if (validView(D))
    deskTop->execView(D);

  destroy(D);
}
/*-----------------------------------------------------------------*/
/*
void TDCApp::readMe()
{
 char* sTmp  =  new char[MAXMSG];
 strcpy2(sTmp, getenv("HOME"));
 strcat2(sTmp, "/ldcc/README");

 if (access(sTmp, R_OK)==0) {
   TFileWindow* FW = new TFileWindow(sTmp);
   if (validView(FW))
     deskTop->execView(FW);
   destroy(FW);
   } else {
   strcat(sTmp,"\n\003not found!");
   messageBox(sTmp, mfError|mfOKButton);
 }
 delete [](char*)sTmp;
}
*/
/*-----------------------------------------------------------------*/
/*
void TDCApp::commands()
{
 char* sTmp =  new char[MAXMSG];

 strcpy2(sTmp, getenv("HOME"));
 strcat2(sTmp, "/ldcc/commands");

 if (access(sTmp, R_OK)==0) {
   TFileWindow* FW = new TFileWindow("");
   if (validView(FW))
     deskTop->execView(FW);
   destroy(FW);
   } else {
   strcat(sTmp,"\n\003not found!");
   messageBox(sTmp, mfError|mfOKButton);
 }
 delete [](char*)sTmp;
}
*/
/*-----------------------------------------------------------------*/
/*
void TDCApp::VAR()
{
 char* sTmp =  new char[MAXMSG];

 strcpy2(sTmp, getenv("HOME"));
 strcat2(sTmp, "/ldcc/VAR");

 if (access(sTmp, R_OK)==0) {
   TFileWindow* FW = new TFileWindow("./dctc/VAR");
   if (validView(FW))
     deskTop->execView(FW);
   destroy(FW);
   } else {
   strcat(sTmp,"\n\003not found!");
   messageBox(sTmp, mfError|mfOKButton);
 }
 delete [](char*)sTmp;
}
*/
/*-----------------------------------------------------------------*/
void TDCApp::shutDown()
{
 DelStr(cfgFile);
 DelStr(debFile);
 DelStr(logFile);
 DelStr(colCfgFile);
 TApplication::shutDown();
}
/*-----------------------------------------------------------------*/



/*-----------------------------------------------------------------*/
//                    SIGNAL Handler
/*-----------------------------------------------------------------*/
void mysig(int v)
{
 delete DCApp;

 char s[21];
 sprintf(s, "SIGNAL: %d",v);
 DEB(s);

 fprintf(stderr, "\nProgram shutdown\nafter SIGNAL: %d\n",v);
 _exit(0);
}
/*-----------------------------------------------------------------*/
void mysig_ign(int v)
{
 return;
}
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
//                            MAIN
/*-----------------------------------------------------------------*/
int main(int varc, char *vars[])
{
 int ch;
 const char opts[] = "h;;help;;v;;version";
 while ((ch=getopt(varc,vars,opts))!=-1) {
   switch (ch) {
     case 'h': printf("LDCC - Linux Direct Connect Client\n");
               printf("options:\n");
               printf("-h | -help\t show this help\n");
               printf("-v | -version\t show version\n");
               return 0;
     case 'v': printf("LDCC version %s (%s)\n", VER_NO, VER_DATE);
               return 0;
   }
 }


 signal(SIGFPE, mysig);
 signal(SIGSEGV, mysig);
//   signal(SIGQUIT, mysig);
 signal(SIGHUP, mysig);
 signal(SIGABRT, mysig);
 signal(SIGPIPE, mysig);
//   signal(SIGTERM, mysig);
//   signal(SIGUSR1, mysig);
//   signal(SIGUSR2, mysig);
 signal(SIGSTKFLT, mysig);
 signal(SIGPWR, mysig);


 DCApp = new TDCApp();
 if (DCApp) {
   DCApp->run();
   delete DCApp;
 }

 return 0;
}
/*-----------------------------------------------------------------*/

