/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * sdir.cc: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "sdir.h"
/*-----------------------------------------------------------------*/
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <stdlib.h>
#include <sys/stat.h>

#define Uses_TDialog
#define Uses_TListBox
#define Uses_MsgBox
#define Uses_TScrollBar
#define Uses_TDeskTop
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TChDirDialog
#include <tv.h>

#include "inifile.h"
#include "options.h"
#include "const.h"
#include "tools.h"
#include "main.h"
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
class TSDir : public TDialog
{
 public:
                TSDir(TRect AR, char* ATitle);
           void shutDown();
           void handleEvent(TEvent& event);
           void Read();
           void Write();
           void Count();
 private:
   TNSStringCollection* kol;
              TListBox* LB;
           TListBoxRec  dkol;
                  char* old_cwd;
   
           void Add();
           void Del();
};
/*-----------------------------------------------------------------*/
class TDDir : public TDialog
{
 public:
                TDDir(TRect AR, char* ATitle);
           void shutDown();
           void handleEvent(TEvent& event);
           void Read();
           void Write();
 private:
   TInputLine  *IL;
   
           void Add();
};

/*-----------------------------------------------------------------*/
void GetDirSize(char *Path, unsigned long &Size);
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
void Shared()
{
  TSDir* D;
 ushort  ret;

 D = new TSDir(TRect(0,0,69,16),"Shared folders");
 ret=DCApp->deskTop->execView(D);
 if (ret==cmOK) {
   D->Write();
 }
 DCApp->destroy(D);
}
/*-----------------------------------------------------------------*/
void Download()
{
  TDDir* D;
 ushort  ret;

 D = new TDDir(TRect(0,0,57,7),"Download directory");
 ret=DCApp->deskTop->execView(D);
 if (ret==cmOK) {
   D->Write();
 }
 DCApp->destroy(D);
}
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
//               class TSDir : public TDialog
/*-----------------------------------------------------------------*/
TSDir::TSDir(TRect AR, char* ATitle)
      :TDialog(AR, ATitle),TWindowInit(TDialog::initFrame)
{
 options |= ofCentered;

 TScrollBar *SB;
 SB = new TScrollBar(TRect(54,2,55,15));
 insert(SB);
 LB = new TListBox(TRect(2,2,54,15),1,SB);
 insert(LB);

 insert(new TButton(TRect(56, 2,67, 4), "~A~dd",    cmAdd, bfNormal));
 insert(new TButton(TRect(56, 4,67, 6), "De~l~ete", cmDel, bfNormal));
 insert(new TButton(TRect(56, 6,67, 8), "~T~otal",  cmCount, bfNormal));

 insert(new TButton(TRect(56,11,67,13), "~O~K", cmOK, bfNormal));
 insert(new TButton(TRect(56,13,67,15), "Cancel", cmCancel, bfNormal));

 old_cwd = NewStr(512);
 getcwd(old_cwd,512);
 kol = NULL;
 Read();
}
/*-----------------------------------------------------------------*/
void TSDir::shutDown()
{
 chdir(old_cwd);
 DelStr(old_cwd);
 TDialog::shutDown();
}
/*-----------------------------------------------------------------*/
void TSDir::handleEvent(TEvent& event)
{
 TDialog::handleEvent(event);
 
 if (event.what==evCommand) {
   switch (event.message.command) {
     case cmAdd:   Add();   break;
     case cmDel:   Del();   break;
     case cmCount: Count(); break;
     default:               return;
   }
   clearEvent(event);
 }
}
/*-----------------------------------------------------------------*/
void TSDir::Count()
{
 unsigned long ts;
     long long ca;
        ushort n,t;
          char s[256];

 WaitON(true, "Counting...");
 ca = 0L;
 t = kol->getCount();
 for (n=0;n<t;n++) {
    WaitUpdate(t,n);
    strcpy2(s, (char*)kol->at(n));
    ts = 0L;
    GetDirSize(s, ts);
    ca += ts;
 }
 WaitOFF();
 sprintf(s,"\003Total size\n\003of shared folders:\n\003%.2lf GB", (double)ca/(1024.0*1024.0*1024.0));
 messageBox(s, mfInformation|mfOKButton);
}
/*-----------------------------------------------------------------*/
void TSDir::Add()
{
 if (kol->getCount()>19) {
   messageBox("\003STOP!\n\003You can share only 20 folders.", mfInformation|mfOKButton);
   return;
 }

 TChDirDialog* D;
 ushort        ret;
 D = new TChDirDialog(cdNormal,0);
 ret = DCApp->deskTop->execView(D);
 if (ret==cmOK) {
   char new_cwd[512];
   getcwd(new_cwd,512);
   kol->insert(NewStr(new_cwd));
   LB->setRange(kol->getCount());
   LB->draw();
 }
 destroy(D);
}
/*-----------------------------------------------------------------*/
void TSDir::Del()
{
 if (kol->getCount()==0) return;

 LB->getData(&dkol);
 kol->atRemove(dkol.selection);
 LB->setRange(kol->getCount());
 LB->focusItem(0);
 LB->draw();
}
/*-----------------------------------------------------------------*/
void TSDir::Read()
{
 TINIfile* ini;
     char  buf1[256*2];
     long  i;
 
 kol            = new TNSStringCollection(5, 5);
 dkol.items     = kol;
 dkol.selection = 0;

 ini = new TINIfile(true);

 for (i=1L;i<21;i++) {
  if ( ini->Read(cfgFile, "Shared", i, buf1, "") ) {
    if (buf1[0]!='\0') {
      kol->insert(NewStr(buf1));
    }
  }
 }
 destroy(ini);

 LB->setData(&dkol);
}
/*-----------------------------------------------------------------*/
void TSDir::Write()
{
 TINIfile *ini;
 char      buf1[256*2];
 long      i;
 
 ini = new TINIfile(true);

 for (i=1L;i<21;i++) {
   if (i<(kol->getCount()+1)) strcpy2(buf1, (char*)kol->at(i-1));
                    else buf1[0]='\0';
   ini->Write(cfgFile,"Shared", i, buf1);
 }
 destroy(ini);
}
/*-----------------------------------------------------------------*/



/*-----------------------------------------------------------------*/
//                 class TDDir : public TDialog
/*-----------------------------------------------------------------*/
TDDir::TDDir(TRect AR, char* ATitle)
      :TDialog(AR, ATitle),TWindowInit(TDialog::initFrame)
{
 options |= ofCentered;

 IL = new TInputLine(TRect(2,2,55,3),80);
 insert(IL);

 insert(new TButton(TRect(2, 4, 13, 6), "~A~dd",  cmAdd,    bfNormal));
 insert(new TButton(TRect(31,4,43,6),   "~O~K",   cmOK,     bfNormal));
 insert(new TButton(TRect(43,4,55,6),   "Cancel", cmCancel, bfNormal));

 Read();
}
/*-----------------------------------------------------------------*/
void TDDir::shutDown()
{
 TDialog::shutDown();
}
/*-----------------------------------------------------------------*/
void TDDir::handleEvent(TEvent &event)
{
 TDialog::handleEvent(event);
 
 if (event.what==evCommand) {
   switch (event.message.command) {
     case cmAdd:   Add();   break;
     default:               return;
   }
   clearEvent(event);
 }
}
/*-----------------------------------------------------------------*/
void TDDir::Add()
{
 TChDirDialog* D;
 ushort        ret;
 char          old_cwd[512];

 getcwd(old_cwd,512);
 
 D = new TChDirDialog(cdNormal,0);
 ret = DCApp->deskTop->execView(D);
 if (ret==cmOK) {
   char new_cwd[512];
   getcwd(new_cwd,512);
   IL->setData(new_cwd);
 }
 destroy(D);

 chdir(old_cwd);
}
/*-----------------------------------------------------------------*/
void TDDir::Read()
{
 TINIfile* ini;
     char  buf1[256*2];
     long  i;
 
 ini = new TINIfile(true);

 char *s_tmp = NewStr(MAXMSG);
 strcpy2(s_tmp, getenv("HOME"));
 strcat2(s_tmp, "/.ldcc");
 ini->Read(cfgFile, "Download", 1, buf1, s_tmp);
 IL->setData(buf1);
 DelStr(s_tmp);
 destroy(ini);
}
/*-----------------------------------------------------------------*/
void TDDir::Write()
{
 TINIfile *ini;
 char      buf1[256*2];
 long      i;
 
 ini = new TINIfile(true);

 IL->getData(buf1);
 ini->Write(cfgFile, "Download", 1, buf1);
 destroy(ini);
 strcpy(downloadDir, buf1);
}
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
void GetDirSize(char *Path, unsigned long &Size)
{
           DIR* dir;
 struct dirent* obj;
   struct stat  fs;
          char  s[215];

 dir=opendir( Path );
 if ( dir!=NULL ) {
   while((obj=readdir(dir))!=NULL){
     if ( (strcmp(obj->d_name,".")!=0) && (strcmp(obj->d_name,"..")!=0) ) {
       sprintf(s, "%s/%s", Path, obj->d_name);
       stat(s, &fs);
       if ( S_ISDIR(fs.st_mode) ) {
           //strcpy(s, Path);
           //strcat(s, obj->d_name);
           //strcat(s, "/");
           GetDirSize(s, Size);
           } else {
           Size += fs.st_size;
       }
     }
   }
 }
 closedir(dir);
}
/*-----------------------------------------------------------------*/

