/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * tools.cc: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "tools.h"
/*-----------------------------------------------------------------*/
#include <string.h>
#include <unistd.h>
#include <time.h>

#define Uses_TKeys
#define Uses_MsgBox
#define Uses_TRect
#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_DrawBuffer
#define Uses_TScrollBar
#include <tv.h>

#include "options.h"
#include "main.h"
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
char* NewStr(char* str)
{
 char* s;
 if (str == NULL ) return (char*)NULL;
 s = new char[strlen(str)+1];
 strcpy(s, str);
 return s;
}
/*-----------------------------------------------------------------------*/
char* NewStr(int size)
{
 char* s;
 if (size == 0 ) return (char*)NULL;
 s = new char[size];
 memset(s, 0x0, size);
 return s;
}
/*-----------------------------------------------------------------------*/
void DelStr(char* str)
{
 if (str == NULL) return;
 delete [](char*)str;
 str = (char*)NULL;
}
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void ConvMsg(char* Msg)
{
 char* p = Msg;

 while (*p!='\0') {
   unsigned char c = (unsigned char)*p;

   for (uchar i=0; i<20; i++) {
     if ( mx[i][0]==c ) *p=(char)mx[i][1];
     i = 21;
   }
   if ((unsigned char)*p<32) *p=32;
   p++;
 }

}
/*-----------------------------------------------------------------*/

/*------------------------------------------------------------------*/
//             class TSrchItem : public TObject
/*------------------------------------------------------------------*/
TSrchItem::TSrchItem():TObject()
{
 Nick = NULL;
 Op   = false;
 Path = NULL;
 File = NULL;
 Size = 0L;
 Slot_Free = Slot_Total = 0;
}
/*------------------------------------------------------------------*/
void TSrchItem::shutDown()
{
 DelStr(Nick);
 DelStr(Path);
 DelStr(File);
 TObject::shutDown();
}
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
//      class TNSStringCollection : public TStringCollection
/*------------------------------------------------------------------*/
TNSStringCollection::TNSStringCollection(ccIndex aLimit, ccIndex aDelta)
                    :TStringCollection(aLimit, aDelta)
{
}
/*------------------------------------------------------------------*/
int TNSStringCollection::compare(void* key1, void* key2)
{
 return (int)-1;
}
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
//          class TObjCollection : public TCollection
/*------------------------------------------------------------------*/
TObjCollection::TObjCollection(ccIndex aLimit, ccIndex aDelta)
               :TCollection(aLimit, aDelta)
{
}
/*------------------------------------------------------------------*/
void* TObjCollection::readItem( ipstream& )
{
 return NULL;
}
/*------------------------------------------------------------------*/
void TObjCollection::writeItem( void *, opstream& )
{
}
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
//       class TMyStaticText : public TStaticText
/*------------------------------------------------------------------*/
TMyStaticText::TMyStaticText(const TRect& bounds, const char* aText)
              :TStaticText(bounds, aText)
{
}
/*------------------------------------------------------------------*/
void TMyStaticText::setText(const char* aText)
{
 delete []text;
 text = NewStr(strlen(aText)+1);
 strcpy((char*)text, aText);
 draw();
}
/*------------------------------------------------------------------*/
char* TMyStaticText::getText()
{
 return (char*)text;
}
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
//                     class TSoctet
/*------------------------------------------------------------------*/
TSocket::TSocket()
{
 memset(&name, 0x0, sizeof(struct sockaddr_un));
 a  = -1;
 fd = -1;
 FileName = NULL;
}
/*------------------------------------------------------------------*/
void TSocket::shutDown()
{
 if (FileName!=NULL) DelStr(FileName);
}
/*------------------------------------------------------------------*/
bool TSocket::Open(char* aFileName, bool silent)
{
 if (FileName!=NULL) DelStr(FileName);
 FileName = NewStr(aFileName);
 
 fd=socket(AF_LOCAL,SOCK_STREAM,0);
 if(fd==-1) {
   if (!silent) messageBox("socket error.", mfError|mfOKButton);
   return false;
 }
 if (fcntl(fd,F_SETFL,O_NONBLOCK)==-1) {
   if (!silent) messageBox("fcntl error", mfError|mfOKButton);
 }

 name.sun_family = AF_LOCAL;
 strcpy2(name.sun_path, FileName);

 a=connect(fd, (struct sockaddr*)&name, sizeof(name));
 if (a!=0) {
   close(fd);
   if (!silent) messageBox("Connect error.", mfError|mfOKButton);
   return false;
 }
 return true;
}
/*------------------------------------------------------------------*/
void TSocket::Close()
{
 if (fd==-1) return;
 close(fd);
 fd = -1;
}
/*------------------------------------------------------------------*/
void TSocket::Read(char* mess)
{
 mess[0]='\0';
 if (fd==-1) return;

 ushort i;
 int    n;
 char   c;

 i=0;
 do {
   n=read(fd,&c,1);
   if (n<1) {
     usleep(1000); // dealy
     return;
   }
   if (i<MAXMSG) {
     mess[i] = c; // truncate too long lines
     i++;
   }
 } while(c!='\n');
 mess[(i>1)?i-2:0]='\0';

 // output to debug log
 if (f_deb!=NULL) {
   if (strlen(mess)>0) {
     DEB2("Read > ", mess);
   }
 }
 
 // wait a moment :-)
 usleep(1000);
}
/*------------------------------------------------------------------*/
void TSocket::Write(char* mess)
{
 DEB2("Write> ", mess);
 if (send(fd, mess, strlen(mess), MSG_DONTWAIT)==-1) {
   messageBox("Send to socket error.", mfError|mfOKButton);
 }
}
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
char *NextItem(char *(&Item))
{
 return NextItemDenom(Item, '|');
}
/*------------------------------------------------------------------*/
char *NextItemSp(char *(&Item))
{
 return NextItemDenom(Item, ' ');
}
/*------------------------------------------------------------------*/
char *NextItemDenom(char *(&Item), char denom)
{
 static char ret[MAXMSG];

 char* p   = Item;
  int  pos = 0;

 memset(ret, 0x0, sizeof(ret));
 
 if (p!=NULL) {
   while ( (*p!=denom)&&(*p!='\0')) {
     ret[pos]= *p;
     if (ret[pos]<32) ret[pos]='_';
     p++;
     pos++;
   }

   if (*p==denom) p++;
 }
 Item = p;
 return (char*)ret;
}
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/*                          TWidokProcent()                              */
/*-----------------------------------------------------------------------*/
TWidokProcent::TWidokProcent(TRect AR)
              :TView(AR)
{              
 // temp - we need only special chars
 TScrollBar* SB;
       char  c1;
       
 SB = new TScrollBar(TRect(0,0,1,5));
 c1 = SB->chars[2];
 c  = SB->chars[3];
 destroy(SB);

 memset(empty, c1, 20);
 empty[20]='\0';
 sprintf(buf, " %3d%% %s", 0, empty);
}
/*-----------------------------------------------------------------------*/
void TWidokProcent::Update(long max, long jest)
{
 uchar pr;
 uchar n, n1;
 long  jest1;
 
 if (max==0) {
   jest=1;
   max=1;
 }
 if (jest>max) jest=max;
 pr = (jest*100+1) / max;
 jest1 = jest*20;
 sprintf(buf, " %3d%% %s", pr, empty);
 n1=6;
 for (n=1; n<=(jest1/ max); n++) {
   buf[n1]=' ';//c;
   n1++;
 }  
 drawView();
}
/*-----------------------------------------------------------------------*/
void TWidokProcent::draw()
{
 TDrawBuffer b;
       uchar color1 = getColor(1);
       uchar color2 = getColor(2);
       uchar len    = strlen(buf);

 b.moveBuf(0, buf, color2, 6);
 b.moveBuf(6, (char*)(buf+6), color1, len-6);
 writeLine(0, 0, len, 1, b);
}
/*-----------------------------------------------------------------------*/
#define cpWidokProcent "\x04\x06"
TPalette& TWidokProcent::getPalette() const
{
 static TPalette palette( cpWidokProcent, sizeof(cpWidokProcent)-1 );
 return palette;
}
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
TDialog*       waitd = NULL;
TWidokProcent* waitp = NULL;
/*-----------------------------------------------------------------------*/
void WaitON(bool Progress, char* Info)
{
 char buf[35];

 if (waitd!=NULL) return;
 
 waitd = new TDialog(TRect(0, 0, 31, 7), "Information");
 waitd->options |= ofCentered;
 
 if ( Info == NULL ) {
   sprintf(buf, "\003%s", "Internal operation.");
   } else {
   if ( strlen(Info)>31 ) Info[31]='\0';
   sprintf(buf, "\003%s", Info);
 }
 waitd->insert(new TStaticText(TRect(1, 2, 30, 3), buf));
 waitd->insert(new TStaticText(TRect(1, 5, 30, 6), "\003Please wait..."));
 
 if (Progress) {
   waitp = new TWidokProcent(TRect(2, 4, 28, 5));
   waitd->insert(waitp);
   } else {
   waitp = NULL;
 }

// if (DCApp->validView(waitd))
 DCApp->insert(waitd);
}
/*-----------------------------------------------------------------------*/
void WaitUpdate(long Max, long Cur)
{
 if ((waitd==NULL) || (waitp==NULL)) return;
 waitp->Update(Max, Cur); 
}
/*-----------------------------------------------------------------------*/
void WaitOFF()
{
 if (waitd==NULL) return;
 
 DCApp->remove(waitd);
 DCApp->destroy(waitd);
 waitd=NULL;
 waitp=NULL;
}
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
//       class TNumberValidator : public TValidator
/*-----------------------------------------------------------------*/
TNumberValidator::TNumberValidator():TValidator()
{
}
/*-----------------------------------------------------------------*/
Boolean TNumberValidator::IsValidInput(char* aChar, Boolean)
{
 return ( ((*aChar>47) && (*aChar<59))?True:False ) ;
}
/*-----------------------------------------------------------------*/
void TNumberValidator::Error()
{

}
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
//       class TEmptyValidator : public TValidator
/*-----------------------------------------------------------------*/
TEmptyValidator::TEmptyValidator():TValidator()
{
}
/*-----------------------------------------------------------------*/
Boolean TEmptyValidator::IsValidInput(char* aChar, Boolean)
{
 return False;
}
/*-----------------------------------------------------------------*/
void TEmptyValidator::Error()
{
}
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------------*/
int strlen2(char* s)
{
 if (s==NULL) return (int)0;
 return strlen(s);
}
/*-----------------------------------------------------------------------*/
char *strcat2(char* s1, char* s2)
{
 if (s1==NULL) return (char*)NULL;
 if (s2==NULL) return (char*)strcat(s1,"");
 return (char*)strcat(s1,s2);
}
/*-----------------------------------------------------------------------*/
char *strcpy2(char* s1, char* s2)
{
 if (s1==NULL) return (char*)NULL;
 if (s2==NULL) return (char*)strcpy(s1, "");
 return (char*)strcpy(s1,s2);
}
/*-----------------------------------------------------------------------*/
int strcmp2(char* s1, char* s2)
{
 if (s1==NULL && s2!=NULL) return (int)1;
 if (s1!=NULL && s2==NULL) return (int)-1;
 if (s1==NULL && s2==NULL) return (int)0;

 return (int)strcmp(s1,s2);
}
/*-----------------------------------------------------------------------*/
int strncmp2(char* s1, char* s2, int i)
{
 if (s1==NULL && s2!=NULL) return (int)1;
 if (s1!=NULL && s2==NULL) return (int)-1;
 if (s1==NULL && s2==NULL) return (int)0;

 return (int)strncmp(s1,s2,i);
}
/*-----------------------------------------------------------------------*/
char* GetTime()
{
 static     tm* TM;
 static time_t  T_T;
 static   char  sTmc[11];

 time(&T_T);
 TM=localtime(&T_T);
 sprintf(sTmc, "%02d:%02d:%02d",
         TM->tm_hour, TM->tm_min, TM->tm_sec);
 return (char *)sTmc;
}
/*-----------------------------------------------------------------------*/

