/*
 * LDCC - a Linux Direct Connect Client
 * Copyright (C) 2003 Marcin Slonicki
 *
 * tools.h: Copyright (C) Marcin Slonicki <marcins@softservice.com.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __TOOLS_H
#define __TOOLS_H
/*-----------------------------------------------------------------*/

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <fcntl.h>

#define Uses_TStaticText
#define Uses_TStringCollection
#define Uses_TNSCollection
#define Uses_TPalette
#define Uses_TValidator
#include <tv.h>

/*-----------------------------------------------------------------*/
#define MAXMSG  4096
/*-----------------------------------------------------------------*/
void ConvMsg(char* Msg);
/*-----------------------------------------------------------------*/
char* NewStr(char* str);
char* NewStr(int size);
void  DelStr(char* str);

char* NextItem(char* (&Item));
char* NextItemSp(char* (&Item));
char* NextItemDenom(char* (&Item), char denom);
/*-----------------------------------------------------------------*/
class TSrchItem : public TObject
{
 public:
                TSrchItem();
           void shutDown();

           char* Nick;
           bool  Op;
           char* Path;
           char* File;
           long  Size;
           uchar Slot_Free;
           uchar Slot_Total;
};
/*-----------------------------------------------------------------*/
class TNSStringCollection : public TStringCollection
{
 public:
               TNSStringCollection(ccIndex aLimit, ccIndex aDelta);
 private:
           int compare(void* key1, void* key2);
};
/*-----------------------------------------------------------------*/
class TObjCollection : public TCollection
{
 public:
               TObjCollection(ccIndex aLimit, ccIndex aDelta);
 private:
           void *readItem( ipstream& );
           void writeItem( void *, opstream& );
};
/*-----------------------------------------------------------------*/
class TMyStaticText : public TStaticText
{
 public:
                         TMyStaticText(const TRect& bounds, const char* aText);
                   void  setText(const char* aText);
                   char *getText();
};
/*-----------------------------------------------------------------*/
class TSocket : public TObject
{
 public:
            TSocket();
       void shutDown();
       bool Open(char* aFileName, bool silent = false);
       void Read(char* mess);
       void Write(char* mess);
       void Close();
 private:
                  char* FileName;
    struct sockaddr_un  name;
                   int  a;
                   int  fd;
};
/*-----------------------------------------------------------------*/
class TWidokProcent : public TView {
  public:
                  TWidokProcent(TRect AR);
            void  draw();
virtual TPalette& getPalette() const;
            void  Update(long max, long jest);
  private:
    char  buf[30];
    char  empty[21];
    char  c;
  };
/*-----------------------------------------------------------------*/
class TNumberValidator : public TValidator
{
  public:
                 TNumberValidator();
 virtual Boolean IsValidInput(char*, Boolean);
            void Error();
};
/*-----------------------------------------------------------------*/
class TEmptyValidator : public TValidator
{
  public:
                 TEmptyValidator();
 virtual Boolean IsValidInput(char*, Boolean);
            void Error();
};
/*-----------------------------------------------------------------*/

void WaitON(bool Progress = false, char* Info = (char*)NULL);
void WaitUpdate(long Max, long Cur);
void WaitOFF();

/*-----------------------------------------------------------------*/
int strlen2(char* s);
char *strcat2(char* s1, char* s2);
char *strcpy2(char* s1, char* s2);
int strcmp2(char* s1, char* s2);
int strncmp2(char* s1, char* s2, int i);

char *GetTime();
/*-----------------------------------------------------------------*/
#endif

